/** @file
Defines Platform DPIN PCD Configuration

@copyright
  "INTEL CONFIDENTIAL
  Copyright 2019 - 2020 Intel Corporation.

  The source code contained or described herein and all documents related to the
  source code ("Material") are owned by Intel Corporation or its suppliers or
  licensors. Title to the Material remains with Intel Corporation or its suppliers
  and licensors. The Material may contain trade secrets and proprietary and
  confidential information of Intel Corporation and its suppliers and licensors,
  and is protected by worldwide copyright and trade secret laws and treaty
  provisions. No part of the Material may be used, copied, reproduced, modified,
  published, uploaded, posted, transmitted, distributed, or disclosed in any way
  without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure or delivery
  of the Materials, either expressly, by implication, inducement, estoppel or
  otherwise. Any license under such intellectual property rights must be
  express and approved by Intel in writing.

  Unless otherwise agreed by Intel in writing, you may not remove or alter
  this notice or any other notice embedded in Materials by Intel or
  Intel's suppliers or licensors in any way."

  This file contains a 'Sample Driver' and is licensed as such under the terms
  of your license agreement with Intel or your vendor. This file may be modified
  by the user, subject to the additional terms of the license agreement.

@par Specification Reference:
**/

#include <Library/GpioLib.h>

#ifndef _PLATFORM_DPIN_CONFIG_H_
#define _PLATFORM_DPIN_CONFIG_H_

#define LABEL_HIGH                                            1
#define LABEL_LOW                                             0
#define MAX_DPIN_EXT_PORT                                     4 // This Macro must be same as PcdMaxDpInExtPortSupported

#pragma pack (push,1)
///
/// Structure for Dp-in GPIO DATA
///
typedef struct {
  UINT8           DpInPortNumber;
  UINT8           DpInSkipGpioChecking;
  UINT16          DpInPresenceDetectLabel;
  GPIO_PAD        DpInGpioPad;
} DPIN_GPIO_DATA;

///
/// Structure for Dp-in GPIO DATA ARRAY
///
typedef struct {
  UINT32           NumberOfDpInPort;
  DPIN_GPIO_DATA   DpInGpioConfig[4];
} DPIN_GPIO_DATA_ARRAY;

///
/// Structure for Dp-in GPIO DATA
///
typedef struct {
  ///< Total number of DpIn Port present on Board.
  UINT8   NumberOfDpInPort;
  ///< Indicate which Dp-In port connection detected. Each BIT stand for one Dp-In port.
  UINT8   DpInPortConnectMap;
  ///< SkipPortDetectionViaGpio.
  ///< 00 = External Gfx Dp-In Port Connect Map Detected through Board GPIO,
  ///< 01: External Gfx Dp-In Port Connect Map either Hardcoded or wiil be detected through someother method.
  UINT8   SkipPortDetectionViaGpio;
} DPIN_PORT_MAP_DATA;
#pragma pack (pop)
#endif
